"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const dismissedCount = [];
const dialogHandler = (dialog) => __awaiter(this, void 0, void 0, function* () {
    dismissedCount.push(dialog.message());
    yield dialog.dismiss();
});
exports.getUrl = (path) => `http://localhost:${puppeteerConfig.server.port}/${path.replace(/^\/+/, '')}`;
// redirect browser console messages to the terminal
exports.consoleHandler = (message) => {
    const type = message.type();
    const text = message.text();
    switch (type) {
        case 'log':
            console.log(text);
            break; // tslint:disable-line:no-console
        case 'debug':
            console.debug(text);
            break; // tslint:disable-line:no-console
        case 'info':
            console.info(text);
            break; // tslint:disable-line:no-console
        case 'error':
            console.error(text);
            break; // tslint:disable-line:no-console
        case 'dir':
            console.dir(text);
            break; // tslint:disable-line:no-console
        case 'dirxml':
            console.dirxml(text);
            break; // tslint:disable-line:no-console
        case 'table':
            console.table(text);
            break; // tslint:disable-line:no-console
        case 'trace':
            console.trace(text);
            break; // tslint:disable-line:no-console
        case 'assert':
            console.assert(text);
            break; // tslint:disable-line:no-console
        case 'profile':
            console.profile(text);
            break; // tslint:disable-line:no-console
        case 'profileEnd':
            console.profileEnd(text);
            break; // tslint:disable-line:no-console
        case 'count':
            console.count(text);
            break; // tslint:disable-line:no-console
        case 'timeEnd':
            console.timeEnd(text);
            break; // tslint:disable-line:no-console
        case 'clear':
            console.clear();
            break; // tslint:disable-line:no-console
        case 'warning':
            console.warn(text);
            break; // tslint:disable-line:no-console
        case 'startGroup':
        case 'startGroupCollapsed':
        case 'endGroup':
            console.info(type, text);
            break; // tslint:disable-line:no-console
    }
};
exports.browserBeforeEach = () => {
    // jest-puppeteer will expose the `page` and `browser` globals to Jest tests.
    if (!browser || !page) {
        throw new Error('Browser has not been started! Did you remember to specify `@jest-environment puppeteer`?');
    }
    page.on('console', exports.consoleHandler);
    page.on('dialog', dialogHandler);
    // page.on('pageerror', async e => {
    //     const newStack = await mapStackTrace(e.message, { isChromeOrEdge: true })
    //     console.error(newStack)
    // })
};
exports.browserAfterEach = () => {
    if (!page.isClosed()) {
        // Node 8 does not have EventListener.off(...)
        page.removeListener('console', exports.consoleHandler);
        page.removeListener('dialog', dialogHandler);
    }
};
exports.browserDismissedDialogs = () => dismissedCount;
//# sourceMappingURL=browserSpecUtils.js.map